@@ -196,7 +196,7 @@ struct _iconv_cache_bucket {
   gchar *key;
   guint32 refcount;
   gboolean used;
-  iconv_t cd;
+  GIConv cd;
 };
 
 static GList *iconv_cache_list;
@@ -233,7 +233,7 @@ iconv_cache_init (void)
  * Returns a pointer to the newly allocated cache bucket.
  **/
 struct _iconv_cache_bucket *
-iconv_cache_bucket_new (const gchar *key, iconv_t cd)
+iconv_cache_bucket_new (const gchar *key, GIConv cd)
 {
   struct _iconv_cache_bucket *bucket;
   
@@ -349,16 +349,24 @@ open_converter (const gchar *to_codeset,
       if (bucket->used)
         {
           cd = g_iconv_open (to_codeset, from_codeset);
-          if (cd == (iconv_t) -1)
+          if (cd == (GIConv) -1)
             goto error;
         }
       else
         {
+	  /* Apparently iconv on Solaris <= 7 segfaults if you pass in
+	   * NULL for anything but inbuf; work around that. (NULL outbuf
+	   * or NULL *outbuf is allowed by Unix98.)
+	   */
+	  gsize inbytes_left = 0;
+	  gchar *outbuf = NULL;
+	  gsize outbytes_left = 0;
+		
           cd = bucket->cd;
           bucket->used = TRUE;
           
           /* reset the descriptor */
-          g_iconv (cd, NULL, NULL, NULL, NULL);
+          g_iconv (cd, NULL, &inbytes_left, &outbuf, &outbytes_left);
         }
       
       bucket->refcount++;
@@ -366,7 +374,7 @@ open_converter (const gchar *to_codeset,
   else
     {
       cd = g_iconv_open (to_codeset, from_codeset);
-      if (cd == (iconv_t) -1)
+      if (cd == (GIConv) -1)
         goto error;
       
       iconv_cache_expire_unused ();
@@ -402,11 +410,11 @@ close_converter (GIConv converter)
 {
   struct _iconv_cache_bucket *bucket;
   const gchar *key;
-  iconv_t cd;
+  GIConv cd;
   
-  cd = (iconv_t) converter;
+  cd = converter;
   
-  if (cd == (iconv_t) -1)
+  if (cd == (GIConv) -1)
     return 0;
   
   G_LOCK (iconv_cache_lock);
@@ -1189,6 +1197,35 @@ g_locale_from_utf8 (const gchar *utf8string,
 #endif /* !G_PLATFORM_WIN32 */
 }
 
+#ifndef G_PLATFORM_WIN32
+static gboolean
+have_broken_filenames (void)
+{
+  static gboolean initialized = FALSE;
+  static gboolean broken;
+  
+  if (initialized)
+    return broken;
+
+  broken = (getenv ("G_BROKEN_FILENAMES") != NULL);
+  
+  initialized = TRUE;
+  
+  return broken;
+}
+#endif /* !G_PLATFORM_WIN32 */
+
+/* This is called from g_thread_init(). It's used to
+ * initialize some static data in a threadsafe way.
+ */
+void 
+g_convert_init (void)
+{
+#ifndef G_PLATFORM_WIN32
+  (void)have_broken_filenames ();
+#endif /* !G_PLATFORM_WIN32 */
+}
+
 /**
  * g_filename_to_utf8:
  * @opsysstring:   a string in the encoding for filenames
@@ -1225,7 +1262,7 @@ g_filename_to_utf8 (const gchar *opsysstring,
 			   error);
 #else  /* !G_PLATFORM_WIN32 */
       
-  if (getenv ("G_BROKEN_FILENAMES"))
+  if (have_broken_filenames ())
     return g_locale_to_utf8 (opsysstring, len,
 			     bytes_read, bytes_written,
 			     error);
@@ -1268,7 +1305,7 @@ g_filename_from_utf8 (const gchar *utf8string,
 			     bytes_read, bytes_written,
 			     error);
 #else  /* !G_PLATFORM_WIN32 */
-  if (getenv ("G_BROKEN_FILENAMES"))
+  if (have_broken_filenames ())
     return g_locale_from_utf8 (utf8string, len,
 			       bytes_read, bytes_written,
 			       error);
